<?php
/**
 *
 * This software is a derived product, based on:
 * @name     	PortaMX-SubForums
 * @copyright	PortaMx Corp. http://portamx.com (SMF Version)
 *
 * and is rewritten for ElkArte:
 * @author	ahrasis (Hj Ahmad Rasyid Hj Ismail)
 * @email	ahrasis@gmail.com
 * @link	http://sahabat.ahrasis.com
 * @license	BSD http://opensource.org/licenses/BSD-3-Clause
 * @addon	MultiTenancy
 *
 **/

if (!defined('ELK'))
	die('Hacking attempt...');

/**
 *Init..
**/
global $MultiTenancy, $base_boardurl, $boardurl, $board_language, $language;

// define the url's
$base_boardurl = $boardurl;
$parts = parse_url($boardurl);
$boardurl = $parts['scheme'] .'://'. $_SERVER['SERVER_NAME'] .(!empty($parts['port']) ? ':'. $parts['port'] : '') . (!empty($parts['path']) ? $parts['path'] : '');
$board_language = $language;

// setup function array
$MultiTenancy = array(
	'isAllowed' => 'MultiTenancy_isAllowed',
	'LoadTheme' => 'MultiTenancy_LoadTheme',
	'getScripturl' => 'MultiTenancy_GetScripturl',
	'getMemberurl' => 'MultiTenancy_GetMemberurl',
	'isOwnurl' => 'MultiTenancy_isOwnurl',
	'checkurl' => 'MultiTenancy_checkurl',
	'updateStats' => 'MultiTenancy_updStats',
);

/**
 *Handle the function calls
**/
function MultiTenancy($addon, $value = null)
{
	global $MultiTenancy;

	if(isset($MultiTenancy[$addon]) && function_exists($MultiTenancy[$addon]))
		return $MultiTenancy[$addon]($value);
}

/**
 *Access to MultiTenancy
* called from index.php
**/
function MultiTenancy_isAllowed()
{
	global $context, $modSettings, $base_boardurl, $boardurl, $user_info, $maintenance, $txt;

	// Calling for MultiTenancy
	if($boardurl != $base_boardurl && isset($modSettings['subforums'][$_SERVER['SERVER_NAME']]))
	{
		// we are on the correct category / board?
		$error = 0;
		if(!empty($_GET['c']) && !in_array($_GET['c'], explode(',', $modSettings['subforums'][$_SERVER['SERVER_NAME']]['cats'])))
			$error = 1;
		elseif(!empty($_GET['topic']) && !empty($context['current_board']) && !in_array($context['current_board'], explode(',', $modSettings['subforums'][$_SERVER['SERVER_NAME']]['boards'])))
			$error = 2;
		elseif(!empty($_GET['board']) && !in_array($_GET['board'], explode(',', $modSettings['subforums'][$_SERVER['SERVER_NAME']]['boards'])))
			$error = 3;

		if(!empty($error))
		{
			$error = sprintf($txt['tenancy_error404'], $txt['tenancy_404types'][$error], $modSettings['subforums'][$_SERVER['SERVER_NAME']]['name']);
			header('HTTP/1.0 404 '. $error);
			header('Content-Type: text/plain; charset=' . (empty($context['character_set']) ? 'ISO-8859-1' : $context['character_set']));
			die('404 - '. $error);
		}

		// check access
		$acs = array_intersect($user_info['groups'], $modSettings['subforums'][$_SERVER['SERVER_NAME']]['groups']);
		if(empty($acs) && !allowedTo('admin_forum'))
		{
			// no access, logout ...
			if(!$user_info['is_guest'])
				MultiTenancy_noAccess($boardurl, $modSettings['subforums'][$_SERVER['SERVER_NAME']]['name']);

				// login...
			elseif(empty($_REQUEST['action']) || (!empty($_REQUEST['action']) && !in_array($_REQUEST['action'], array('login2', 'register', 'register2', 'reminder'))))
				MultiTenancy_Login(false);

			// logout requested ?
			elseif(!empty($_REQUEST['action']) && $_REQUEST['action'] == 'logout')
				$_SESSION['logout_url'] = $boardurl;
		}
	}

	// main forum only for admins ?
	elseif(empty($maintenance) && empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]) && !empty($modSettings['subforums_adminonly']) && !allowedTo('admin_forum'))
		MultiTenancy_Login(true);
}

/**
 *load the Theme
* called from Load.php (LoadTheme)
**/
function MultiTenancy_LoadTheme($id_theme)
{
	global $modSettings, $user_info, $board_language, $language;

	if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['theme']))
		$id_theme = (int) $modSettings['subforums'][$_SERVER['SERVER_NAME']]['theme'];

	// to NOT overwrite users language comment or remove the next five lines
	if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['language']))
	{
		$language = $board_language;
		$user_info['language'] = $modSettings['subforums'][$_SERVER['SERVER_NAME']]['language'];
	}

	return $id_theme;
}

/**
 *check if the Scripturl own
* called from News.controller.php xml (fix_possible_url)
**/
function MultiTenancy_isOwnurl($val)
{
	global $modSettings;

	foreach($modSettings['subforums'] as $hostname => $data)
		if($hostname == $val)
			return $val;

	return '';
}

/**
 *check if on the current forum
* called from Who and Subs-MemberOnline
**/
function MultiTenancy_checkurl($url)
{
	global $modSettings, $base_boardurl;

	$parts = parse_url($base_boardurl);
	if(!empty($url['host']) && array_key_exists($url['host'], $modSettings['subforums']))
		return $url['host'];

	return $parts['host'];
}

/**
 *Update the total Posts/Topics
* called from Subs.php
**/
function MultiTenancy_updStats($change)
{
	global $modSettings;
	
	$db = database();

	if(array_key_exists($_SERVER['SERVER_NAME'], $modSettings['subforums']))
	{
		$host = $_SERVER['SERVER_NAME'];

		foreach($change as $variable => $value)
		{
			if($value !== true)
			{
				// Get the number of messages / topics
				$col = str_replace('total', '', $variable);
				$result = $db->query('', '
					SELECT SUM(num'. $col .' + unapproved'. $col .') AS total'. $col .'
					FROM {db_prefix}boards
					WHERE redirect = {string:blank_redirect} AND id_cat IN ({raw:cats})'. (!empty($modSettings['recycle_enable']) && $modSettings['recycle_board'] > 0 ? '
						AND id_board != {int:recycle_board}' : ''),
					array(
						'cats' => $modSettings['subforums'][$host]['cats'],
						'recycle_board' => isset($modSettings['recycle_board']) ? $modSettings['recycle_board'] : 0,
						'blank_redirect' => '',
					)
				);
				$row = $db->fetch_assoc($result);
				$db->free_result($result);
				$change[$variable] = $row[$variable] === null ? 0 : $row[$variable];
			}

			foreach($change as $variable => $value)
			{
				$db->query('', '
					UPDATE {db_prefix}subforums
					SET {raw:variable} = {'. ($value === false || $value === true ? 'raw' : 'int') . ':value}
					WHERE forum_host = {string:host}',
					array(
						'value' => $value === true ? $variable .' + 1' : ($value === false ? $variable .' - 1' : $value),
						'variable' => $variable,
						'host' => $host,
					)
				);
				$modSettings['subforums'][$host][$variable] = $value === true ? $modSettings['subforums'][$host][$variable] + 1 : ($value === false ? $modSettings['subforums'][$host][$variable] - 1 : $value);
			}
			cache_put_data('ahrasis-', NULL, 120);
		}
	}
}

/**
 *Get the Scripturl for MultiTenancy
* called from News.php xml (news, recent)
**/
function MultiTenancy_GetScripturl($data)
{
	global $modSettings, $base_boardurl, $host;

	list($type, $board, $url) = $data;
	$newurl = '';

	// Find MultiTenancy by category or board
	foreach($modSettings['subforums'] as $host => $data)
	{
		$boards = explode(',', $data[$type]);
		array_walk($boards, function (&$v, $k) { $v = intval(trim($v)); });
		if(in_array(intval($board), $boards))
		{
			// found it.. return the new url
			$tmp = parse_url($base_boardurl);
			return str_replace($tmp['host'], $host, $url);
		}
	}

	// main RSS not disabled .. return the url
	if(empty($modSettings['subforums_xmlrss']))
		return $url;

	// nothing found
	return false;
}

/**
 *Check MultiTenancy member access
* called from News.php xml (memberlist)
**/
function MultiTenancy_GetMemberurl($url)
{
	global $modSettings, $base_boardurl;

	// main forum accessible .. return the url
	if(empty($modSettings['subforums_adminonly']))
		return $url;

	// Find MultiTenancy with guest/member access and return the url
	else
	{
		$base = parse_url($base_boardurl);
		foreach($modSettings['subforums'] as $host => $data)
		{
			if(in_array(0, $data['groups']) || in_array(-1, $data['groups']))
				return str_replace($base['host'], $host, $url);
		}
	}

	// nothing found
	return false;
}

/**
 *check hostname
* called from PortaMx LoadData.php (custom actions)
**/
function MultiTenancy_checkhost($itemData, &$bits)
{
	// Find MultiTenancy host
	if(preg_match('~\[host\=([a-zA-Z0-9\.\-\_\*\?\,\^\s]+)\]~i', $itemData, $temp) > 0 && isset($temp[1]))
	{
		$stripbits = array('spage', 'art', 'cat', 'child');
		$state = pmx_getBits($bits, $stripbits);
		$found = false;
		$hosts = explode(',', $temp[1]);

		foreach($hosts as $hostname)
		{
			$hostname = trim($hostname);
			$host = preg_replace('/^\^/', '', $hostname);
			$isNot = $host != $hostname;

			// mask out wildcards * and ?
			$host = str_replace(array('*','?'), array('.*','.?'), trim($host));
			if(preg_match('~'. $host .'~i', $_SERVER['SERVER_NAME'], $match) != 0 && $match[0] == $_SERVER['SERVER_NAME'])
			{
				// found it..
				$found = true;
				if(!empty($isNot))
					$bits = pmx_setBits(0);
				elseif((is_null($state) || (empty($bits['action']) && !empty($state))))
					$bits['action'] = intval(empty($isNot));
				break;
			}
		}

		if(empty($found))
		{
			if((is_null($state) || (empty($bits['action']) && !empty($state))))
				$bits['action'] = 1;
		}
	}
}

/**
 *MultiTenancy Login
**/
function MultiTenancy_Login($adm_mode)
{
	global $context, $user_info, $user_profile;
	
	$db = database();

	// save the current url for login
	$_SESSION['login_url'] = htmlspecialchars__recursive($_SERVER['QUERY_STRING']);

	// require_once(CONTROLLERDIR . '/Auth.controller.php');

	// normal login ?
	if(empty($adm_mode) && empty($_REQUEST['action']) && empty($_REQUEST['board']) && empty($_REQUEST['topic']))
		$_REQUEST = $_GET = array('action' => 'login');

	// admin login ?
	if($adm_mode)
	{
		if(!empty($_REQUEST['action']) && in_array($_REQUEST['action'], array('reminder', 'login2')))
		{
			// missing hash / username or is reminder ...
			if(empty($_REQUEST['hash_passwrd']) || empty($_REQUEST['user']) || $_REQUEST['action'] == 'reminder')
			{
				if($_REQUEST['action'] == 'reminder')
					$_SESSION['failed_login'] = 0;
				$_REQUEST = $_GET = array();
			}

			// login failed..
			elseif(!empty($_SESSION['subforums_failed_login']))
				$_REQUEST = $_GET = array();

			else
			{
				// get memberdate ...
				if($mem = loadMemberData($_REQUEST['user'], true, 'profile'))
				{
					// get all groups for this member
					$mem = current($mem);
					$groups = array('-1');
					if(empty($user_profile[$mem]['id_group']) && empty($user_profile[$mem]['additional_groups']))
						$_REQUEST = $_GET = array();

					// member have groups..
					else
					{
						$groups = explode(',', $user_profile[$mem]['id_group']);
						if(!empty($user_profile[$mem]['additional_groups']))
							$groups = array_merge($groups, explode(',', $user_profile[$mem]['additional_groups']));

						// get permissions for the group(s)
						$perms = 0;
						$request = $db->query('', '
							SELECT count(permission)
							FROM {db_prefix}permissions
							WHERE id_group IN ({array_int:group}) AND permission = {string:perm} AND add_deny = 1',
							array(
								'group' => $groups,
								'perm' => 'admin_forum'
							)
						);
						list($perms) = $db->fetch_row($request);
						$db->free_result($request);

						// no permissions and not group 1 .. failed admin login
						if(empty($perms) && !in_array(1, $groups))
							$_REQUEST = $_GET = array();
					}
				}

				// member not found .. failed admin login
				else
					$_REQUEST = $_GET = array();
			}
		}

		// put css on header to hide the Maintenace header
		$context['html_headers'] .= '
		<style type="text/css">
		.subforum_hide { display:none }
		</style>';

		// achow the admin login
		$_SESSION['subforums_failed_login'] = false;
		maintenance_mode();
	}

	// member login
	else
		kickguest();
}

/**
 *MultiTenancy login failed
**/
function MultiTenancy_noAccess($url, $subfName)
{
	global $context;
	
	if(!empty($url))
		$_SESSION['logout_url'] = $url;

	$_REQUEST = $_GET = array('action' => 'logout');

	require_once(CONTROLLERDIR . '/Auth.controller.php');
	logout($internal = true, $redirect = false);

	// show a error message
	// fatal_lang_error('tenancy_nopermission', false, $subfName);
}

function logout($internal = false, $redirect = true)
{
	global $user_info, $user_settings;

	// Make sure they aren't being auto-logged out.
	if (!$internal)
		checkSession('get');

	require_once(SUBSDIR . '/Auth.subs.php');

	if (isset($_SESSION['pack_ftp']))
		$_SESSION['pack_ftp'] = null;

	// They cannot be open ID verified any longer.
	if (isset($_SESSION['openid']))
		unset($_SESSION['openid']);

	// It won't be first login anymore.
	unset($_SESSION['first_login']);

	// Just ensure they aren't a guest!
	if (!$user_info['is_guest'])
	{
		// Pass the logout information to integrations.
		call_integration_hook('integrate_logout', array($user_settings['member_name']));

		// If you log out, you aren't online anymore :P.
		require_once(SUBSDIR . '/Logging.subs.php');
		logOnline($user_info['id'], false);
	}

	// Logout? Let's kill the admin/moderate/other sessions, too.
	$types = array('admin', 'moderate');
	call_integration_hook('integrate_validateSession', array(&$types));
	foreach ($types as $type)
		unset($_SESSION[$type . '_time']);

	$_SESSION['log_time'] = 0;

	// Empty the cookie! (set it in the past, and for id_member = 0)
	setLoginCookie(-3600, 0);

	// And some other housekeeping while we're at it.
	session_destroy();
	if (!empty($user_info['id']))
	{
		$tokenizer = new Token_Hash();
		require_once(SUBSDIR . '/Members.subs.php');
		updateMemberData($user_info['id'], array('password_salt' => $tokenizer->generate_hash(4)));
	}

	// Off to the merry board index we go!
	if ($redirect)
	{
		if (empty($_SESSION['logout_url']))
			redirectexit('', detectServer()->is('needs_login_fix'));
		elseif (!empty($_SESSION['logout_url']) && (substr($_SESSION['logout_url'], 0, 7) !== 'http://' && substr($_SESSION['logout_url'], 0, 8) !== 'https://'))
		{
			unset($_SESSION['logout_url']);
			redirectexit();
		}
		else
		{
			$temp = $_SESSION['logout_url'];
			unset($_SESSION['logout_url']);

			redirectexit($temp, detectServer()->is('needs_login_fix'));
		}
	}
}

function kickguest()
{
	global $txt, $context;

	loadLanguage('Login');
	loadTemplate('Login');
	loadJavascriptFile('sha256.js', array('defer' => true));
	createToken('login');

	// Never redirect to an attachment
	if (strpos($_SERVER['REQUEST_URL'], 'dlattach') === false)
		$_SESSION['login_url'] = $_SERVER['REQUEST_URL'];

	$context['sub_template'] = 'kick_guest';
	$context['page_title'] = $txt['login'];
}

function maintenance_mode()
{
	global $txt, $mtitle, $mmessage, $context;

	loadLanguage('Login');
	loadTemplate('Login');
	loadJavascriptFile('sha256.js', array('defer' => true));
	createToken('login');

	// Send a 503 header, so search engines don't bother indexing while we're in maintenance mode.
	header('HTTP/1.1 503 Service Temporarily Unavailable');

	// Basic template stuff..
	$context['sub_template'] = 'maintenance';
	$context['title'] = &$mtitle;
	$context['description'] = &$mmessage;
	$context['page_title'] = $txt['maintain_mode'];
}

class Tenancy
{
	/**
	*
	* Load the settings through integrate_pre_load
	* in Load.php
	*
	**/
	public static function PreLoad()
	{
		global $boardurl, $modSettings, $language, $txt;
		
		$db = database();

		// check if cached ..
		if(empty($modSettings['cache_enable']) || ($modSettings['subforums'] = cache_get_data('ahrasis-', 10)) === null)
		{
			// not cached .. get from database
			$modSettings['subforums'] = array();
			$request = $db->query('', '
				SELECT s.id, s.forum_host, s.forum_name, s.cat_order, s.id_theme, s.language, s.acs_groups, s.reg_group, s.total_posts, s.total_topics, b.id_board
				FROM {db_prefix}subforums AS s
				LEFT JOIN {db_prefix}boards as b ON (FIND_IN_SET(b.id_cat, s.cat_order) > 0)
				ORDER BY b.id_cat, b.id_board',
				array()
			);
			if($db->num_rows($request) > 0)
			{
				while($row = $db->fetch_assoc($request))
				{
					if(!isset($modSettings['subforums'][$row['forum_host']]['cats']))
					{
						$modSettings['subforums'][$row['forum_host']]['name'] = $row['forum_name'];
						$modSettings['subforums'][$row['forum_host']]['cats'] = $row['cat_order'];
						$modSettings['subforums'][$row['forum_host']]['theme'] = $row['id_theme'];
						$modSettings['subforums'][$row['forum_host']]['language'] = $row['language'];
						$modSettings['subforums'][$row['forum_host']]['groups'] = explode(',', $row['acs_groups']);
						$modSettings['subforums'][$row['forum_host']]['reggroup'] = $row['reg_group'];
						$modSettings['subforums'][$row['forum_host']]['boards'] = array();
						$modSettings['subforums'][$row['forum_host']]['total_posts'] = $row['total_posts'];
						$modSettings['subforums'][$row['forum_host']]['total_topics'] = $row['total_topics'];
					}
					$modSettings['subforums'][$row['forum_host']]['boards'][] = $row['id_board'];
				}
				$db->free_result($request);

				// convert board
				foreach($modSettings['subforums'] as $host => $d)
					$modSettings['subforums'][$host]['boards'] = implode(',', $modSettings['subforums'][$host]['boards']);
			}

			// save it to cache
			if(!empty($modSettings['cache_enable']))
				cache_put_data('ahrasis-', $modSettings['subforums'], 3600);
		}

		// Set language MultiTenancy?
		if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['language']))
			$language = $modSettings['subforums'][$_SERVER['SERVER_NAME']]['language'];

		// load language
		loadLanguage('tenancy/Tenancy');
	}

	/**
	*
	* Initialize forum theme through 
	* integrate_init_theme in Load.php
	*
	**/ /*
	public static function Theme($id_theme)
	{
		global $modSettings, $user_info, $board_language, $language;
		// Initialize Theme
		if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['theme']))
			$id_theme = (int) $modSettings['subforums'][$_SERVER['SERVER_NAME']]['theme'];
		// to NOT overwrite users language comment or remove the next five lines
		if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['language']))
		{
			$language = $board_language;
			$user_info['language'] = $modSettings['subforums'][$_SERVER['SERVER_NAME']]['language'];
		}
		return $id_theme;
	}

	/**
	*
	* Initialize forum theme through 
	* integrate_init_theme in Load.php
	*
	**/ /*
	public static function PreLogStats()
	{
		global $no_stat_actions;
		// Do some logging, unless this is an attachment, avatar, toggle of editor buttons, theme option, XML feed etc.
		if (empty($_REQUEST['action']) || !in_array($_REQUEST['action'], $no_stat_actions))
		{
			// call MultiTenancy check access.
			MultiTenancy('isAllowed');
		}
	}

	/**
	*
	* Initialize forum theme through 
	* integrate_init_theme in Load.php
	*
	**/ /*
	public static function InitTheme($id_theme)
	{
		$id_theme = MultiTenancy('LoadTheme', $id_theme);
	}

	/**
	* 
	* Reset forum name and default directory 
	* through integrate_load_theme in Load.php
	*
	**/
	public static function LoadTheme()
	{
		global $context, $modSettings, $settings;
		// Reset forum name
		if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['name'])) { 
			$context['forum_name_html_safe'] = $context['forum_name'] = $modSettings['subforums'][$_SERVER['SERVER_NAME']]['name'];
		}
	}

	/**
	 *
	 * We don't want any fixed url in the combined css
	 * file(s), we return it back to default i.e. ".."
	 *
	 **/

	public static function ChangeUrlInHive()
	{
		global $context, $settings;
		$parse = parse_url($settings['default_theme_url']);
		foreach ($context['css_files'] as $key => $file) {
			$context['css_files'][$key]['options']['url'] = '..' . $parse['path'];
		}
	}

	/**
	 *
	 * Now we make sure the loaded urls are all the same by
	 * buffering them with string replace. This is to ensure
	 * SSL and font awesome to work properly.
	 *
	 **/

	public static function BufferUrl($buffer)
	{
		global $modSettings, $boardurl, $context, $settings;

		// Check for unwanted before proceeding further
		if (isset($_REQUEST['xml']) || $context['current_action'] == 'printpage')
			return $buffer; // Remove or comment both if not wanted

		$mesfa = array();

		// Get default url and change it to MultiTenancy url
		$default = parse_url($settings['default_theme_url']);
		$mesfa[$default['host']] = $_SERVER['SERVER_NAME'];

		// Get actual url and change it to MultiTenancy url too
		$actual = parse_url($settings['actual_theme_url']);
		$mesfa[$actual['host']] = $_SERVER['SERVER_NAME'];

		// SSL and non SSL url fixes each other
		$scheme = 'http://' . $_SERVER['SERVER_NAME'];
		$ssl_scheme = 'https://' . $_SERVER['SERVER_NAME'];

		if (empty($_SERVER['HTTPS']))
		{ $mesfa[$ssl_scheme] = $scheme; }

		else
		{ $mesfa[$scheme] = $ssl_scheme; }

		// Now let's change necessary things
		return str_replace(array_keys($mesfa), array_values($mesfa), $buffer);
	}

	/**
	* Setup membergroup on Register account
	* Called from hook integrate_register
	**/
	public static function Register(&$regOptions, &$theme_vars)
	{
		global $modSettings, $regOptions;

		if(!empty($modSettings['subforums'][$_SERVER['SERVER_NAME']]['reggroup']) && $modSettings['subforums'][$_SERVER['SERVER_NAME']]['reggroup'] != -1)
			$regOptions['register_vars']['id_group'] = $modSettings['subforums'][$_SERVER['SERVER_NAME']]['reggroup'];
	}

	/**
	* Add Admin menu context
	* Called from hook integrate_admin_areas
	**/
	public static function AdminMenu(&$menudata)
	{
		global $txt;

		// Insert MultiTenancy in admin menu
		$menudata['layout']['areas'] = array_merge(
			array(
				'multitenancy' => array(
					'label' => $txt['tenancy_admin'],
					'dir' => SOURCEDIR,
					'file' => 'addons/tenancy/Tenancy.admin.php',
					'function' => 'MultiTenancyAdmin',
					'icon' => 'languages.png',
					'subsections' => array(
					),
				),
			),
			$menudata['layout']['areas']
		);
	}
}
?>